//
// File:   video_decoder_sample.v
// Date:   31-Oct-05
// Author: J. Castro (MIT 6.111 Fall 2005)
//
// Sample top-level file for the MIT 6.111 labkit, demonstrating
// b&w NTSC video input from the ADV7185.  The video is digitized
// and made available, together with sync signals from the ntsc_decode 
// module.  This sample demo displays a 256 x 192 pixel black and white 
// image undersampled subset of the video in a SVGA (1024 x 768) window.
//
// Switch[0] is used for testing: 0 = digitized video, 1 = b&w test bars
// button_enter is reset
//
// The dot LEDs display the a version number, and the
// 8 discrete LEDs display the video data read from RAM.

`include "video_decoder.v"
`include "display_16hex.v"
`include "videoram.v"

///////////////////////////////////////////////////////////////////////////////
//
// 6.111 FPGA Labkit -- Template Toplevel Module
//
// For Labkit Revision 004
//
//
// Created: October 31, 2004, from revision 003 file
// Author: Nathan Ickes
//
///////////////////////////////////////////////////////////////////////////////
//
// CHANGES FOR BOARD REVISION 004
//
// 1) Added signals for logic analyzer pods 2-4.
// 2) Expanded "tv_in_ycrcb" to 20 bits.
// 3) Renamed "tv_out_data" to "tv_out_i2c_data" and "tv_out_sclk" to
//    "tv_out_i2c_clock".
// 4) Reversed disp_data_in and disp_data_out signals, so that "out" is an
//    output of the FPGA, and "in" is an input.
//
// CHANGES FOR BOARD REVISION 003
//
// 1) Combined flash chip enables into a single signal, flash_ce_b.
//
// CHANGES FOR BOARD REVISION 002
//
// 1) Added SRAM clock feedback path input and output
// 2) Renamed "mousedata" to "mouse_data"
// 3) Renamed some ZBT memory signals. Parity bits are now incorporated into 
//    the data bus, and the byte write enables have been combined into the
//    4-bit ram#_bwe_b bus.
// 4) Removed the "systemace_clock" net, since the SystemACE clock is now
//    hardwired on the PCB to the oscillator.
//
///////////////////////////////////////////////////////////////////////////////
//
// Complete change history (including bug fixes)
//
// 2005-Sep-09: Added missing default assignments to "ac97_sdata_out",
//              "disp_data_out", "analyzer[2-3]_clock" and
//              "analyzer[2-3]_data".
//
// 2005-Jan-23: Reduced flash address bus to 24 bits, to match 128Mb devices
//              actually populated on the boards. (The boards support up to
//              256Mb devices, with 25 address lines.)
//
// 2004-Oct-31: Adapted to new revision 004 board.
//
// 2004-May-01: Changed "disp_data_in" to be an output, and gave it a default
//              value. (Previous versions of this file declared this port to
//              be an input.)
//
// 2004-Apr-29: Reduced SRAM address busses to 19 bits, to match 18Mb devices
//              actually populated on the boards. (The boards support up to
//              72Mb devices, with 21 address lines.)
//
// 2004-Apr-29: Change history started
//
///////////////////////////////////////////////////////////////////////////////

module labkit (beep, audio_reset_b, ac97_sdata_out, ac97_sdata_in, ac97_synch,
	       ac97_bit_clock,
	       
	       vga_out_red, vga_out_green, vga_out_blue, vga_out_sync_b,
	       vga_out_blank_b, vga_out_pixel_clock, vga_out_hsync,
	       vga_out_vsync,

	       tv_out_ycrcb, tv_out_reset_b, tv_out_clock, tv_out_i2c_clock,
	       tv_out_i2c_data, tv_out_pal_ntsc, tv_out_hsync_b,
	       tv_out_vsync_b, tv_out_blank_b, tv_out_subcar_reset,

	       tv_in_ycrcb, tv_in_data_valid, tv_in_line_clock1,
	       tv_in_line_clock2, tv_in_aef, tv_in_hff, tv_in_aff,
	       tv_in_i2c_clock, tv_in_i2c_data, tv_in_fifo_read,
	       tv_in_fifo_clock, tv_in_iso, tv_in_reset_b, tv_in_clock,

	       ram0_data, ram0_address, ram0_adv_ld, ram0_clk, ram0_cen_b,
	       ram0_ce_b, ram0_oe_b, ram0_we_b, ram0_bwe_b, 

	       ram1_data, ram1_address, ram1_adv_ld, ram1_clk, ram1_cen_b,
	       ram1_ce_b, ram1_oe_b, ram1_we_b, ram1_bwe_b,

	       clock_feedback_out, clock_feedback_in,

	       flash_data, flash_address, flash_ce_b, flash_oe_b, flash_we_b,
	       flash_reset_b, flash_sts, flash_byte_b,

	       rs232_txd, rs232_rxd, rs232_rts, rs232_cts,

	       mouse_clock, mouse_data, keyboard_clock, keyboard_data,

	       clock_27mhz, clock1, clock2,

	       disp_blank, disp_data_out, disp_clock, disp_rs, disp_ce_b,
	       disp_reset_b, disp_data_in,

	       button0, button1, button2, button3, button_enter, button_right,
	       button_left, button_down, button_up,

	       switch,

	       led,
	       
	       user1, user2, user3, user4,
	       
	       daughtercard,

	       systemace_data, systemace_address, systemace_ce_b,
	       systemace_we_b, systemace_oe_b, systemace_irq, systemace_mpbrdy,
	       
	       analyzer1_data, analyzer1_clock,
 	       analyzer2_data, analyzer2_clock,
 	       analyzer3_data, analyzer3_clock,
 	       analyzer4_data, analyzer4_clock);

   output beep, audio_reset_b, ac97_synch, ac97_sdata_out;
   input  ac97_bit_clock, ac97_sdata_in;
   
   output [7:0] vga_out_red, vga_out_green, vga_out_blue;
   output vga_out_sync_b, vga_out_blank_b, vga_out_pixel_clock,
	  vga_out_hsync, vga_out_vsync;

   output [9:0] tv_out_ycrcb;
   output tv_out_reset_b, tv_out_clock, tv_out_i2c_clock, tv_out_i2c_data,
	  tv_out_pal_ntsc, tv_out_hsync_b, tv_out_vsync_b, tv_out_blank_b,
	  tv_out_subcar_reset;
   
   input  [19:0] tv_in_ycrcb;
   input  tv_in_data_valid, tv_in_line_clock1, tv_in_line_clock2, tv_in_aef,
	  tv_in_hff, tv_in_aff;
   output tv_in_i2c_clock, tv_in_fifo_read, tv_in_fifo_clock, tv_in_iso,
	  tv_in_reset_b, tv_in_clock;
   inout  tv_in_i2c_data;
        
   inout  [35:0] ram0_data;
   output [18:0] ram0_address;
   output ram0_adv_ld, ram0_clk, ram0_cen_b, ram0_ce_b, ram0_oe_b, ram0_we_b;
   output [3:0] ram0_bwe_b;
   
   inout  [35:0] ram1_data;
   output [18:0] ram1_address;
   output ram1_adv_ld, ram1_clk, ram1_cen_b, ram1_ce_b, ram1_oe_b, ram1_we_b;
   output [3:0] ram1_bwe_b;

   input  clock_feedback_in;
   output clock_feedback_out;
   
   inout  [15:0] flash_data;
   output [23:0] flash_address;
   output flash_ce_b, flash_oe_b, flash_we_b, flash_reset_b, flash_byte_b;
   input  flash_sts;
   
   output rs232_txd, rs232_rts;
   input  rs232_rxd, rs232_cts;

   input  mouse_clock, mouse_data, keyboard_clock, keyboard_data;

   input  clock_27mhz, clock1, clock2;

   output disp_blank, disp_clock, disp_rs, disp_ce_b, disp_reset_b;  
   input  disp_data_in;
   output  disp_data_out;
   
   input  button0, button1, button2, button3, button_enter, button_right,
	  button_left, button_down, button_up;
   input  [7:0] switch;
   output [7:0] led;

   inout [31:0] user1, user2, user3, user4;
   
   inout [43:0] daughtercard;

   inout  [15:0] systemace_data;
   output [6:0]  systemace_address;
   output systemace_ce_b, systemace_we_b, systemace_oe_b;
   input  systemace_irq, systemace_mpbrdy;

   output [15:0] analyzer1_data, analyzer2_data, analyzer3_data, 
		 analyzer4_data;
   output analyzer1_clock, analyzer2_clock, analyzer3_clock, analyzer4_clock;

   ////////////////////////////////////////////////////////////////////////////
   //
   // I/O Assignments
   //
   ////////////////////////////////////////////////////////////////////////////
   
   // Audio Input and Output
   assign beep= 1'b0;
   assign audio_reset_b = 1'b0;
   assign ac97_synch = 1'b0;
   assign ac97_sdata_out = 1'b0;
   // ac97_sdata_in is an input

/*
   // VGA Output
   assign vga_out_red = 10'h0;
   assign vga_out_green = 10'h0;
   assign vga_out_blue = 10'h0;
   assign vga_out_sync_b = 1'b1;
   assign vga_out_blank_b = 1'b1;
   assign vga_out_pixel_clock = 1'b0;
   assign vga_out_hsync = 1'b0;
   assign vga_out_vsync = 1'b0;
*/
   // Video Output
   assign tv_out_ycrcb = 10'h0;
   assign tv_out_reset_b = 1'b0;
   assign tv_out_clock = 1'b0;
   assign tv_out_i2c_clock = 1'b0;
   assign tv_out_i2c_data = 1'b0;
   assign tv_out_pal_ntsc = 1'b0;
   assign tv_out_hsync_b = 1'b1;
   assign tv_out_vsync_b = 1'b1;
   assign tv_out_blank_b = 1'b1;
   assign tv_out_subcar_reset = 1'b0;
   
   // Video Input
   //assign tv_in_i2c_clock = 1'b0;
   assign tv_in_fifo_read = 1'b1;
   assign tv_in_fifo_clock = 1'b0;
   assign tv_in_iso = 1'b1;
   //assign tv_in_reset_b = 1'b0;
   assign tv_in_clock = clock_27mhz;//1'b0;
   //assign tv_in_i2c_data = 1'bZ;
   // tv_in_ycrcb, tv_in_data_valid, tv_in_line_clock1, tv_in_line_clock2, 
   // tv_in_aef, tv_in_hff, and tv_in_aff are inputs
   
   // SRAMs
   assign ram0_data = 36'hZ;
   assign ram0_address = 19'h0;
   assign ram0_adv_ld = 1'b0;
   assign ram0_clk = 1'b0;
   assign ram0_cen_b = 1'b1;
   assign ram0_ce_b = 1'b1;
   assign ram0_oe_b = 1'b1;
   assign ram0_we_b = 1'b1;
   assign ram0_bwe_b = 4'hF;
   assign ram1_data = 36'hZ; 
   assign ram1_address = 19'h0;
   assign ram1_adv_ld = 1'b0;
   assign ram1_clk = 1'b0;
   assign ram1_cen_b = 1'b1;
   assign ram1_ce_b = 1'b1;
   assign ram1_oe_b = 1'b1;
   assign ram1_we_b = 1'b1;
   assign ram1_bwe_b = 4'hF;
   assign clock_feedback_out = 1'b0;
   // clock_feedback_in is an input
   
   // Flash ROM
   assign flash_data = 16'hZ;
   assign flash_address = 24'h0;
   assign flash_ce_b = 1'b1;
   assign flash_oe_b = 1'b1;
   assign flash_we_b = 1'b1;
   assign flash_reset_b = 1'b0;
   assign flash_byte_b = 1'b1;
   // flash_sts is an input

   // RS-232 Interface
   assign rs232_txd = 1'b1;
   assign rs232_rts = 1'b1;
   // rs232_rxd and rs232_cts are inputs

   // PS/2 Ports
   // mouse_clock, mouse_data, keyboard_clock, and keyboard_data are inputs

   // LED Displays
/*
   assign disp_blank = 1'b1;
   assign disp_clock = 1'b0;
   assign disp_rs = 1'b0;
   assign disp_ce_b = 1'b1;
   assign disp_reset_b = 1'b0;
   assign disp_data_out = 1'b0;
 */
   // disp_data_in is an input

   // Buttons, Switches, and Individual LEDs
   //assign led = 8'hFF;
   // button0, button1, button2, button3, button_enter, button_right,
   // button_left, button_down, button_up, and switches are inputs

   // User I/Os
   assign user1 = 32'hZ;
   assign user2 = 32'hZ;
   assign user3 = 32'hZ;
   assign user4 = 32'hZ;

   // Daughtercard Connectors
   assign daughtercard = 44'hZ;

   // SystemACE Microprocessor Port
   assign systemace_data = 16'hZ;
   assign systemace_address = 7'h0;
   assign systemace_ce_b = 1'b1;
   assign systemace_we_b = 1'b1;
   assign systemace_oe_b = 1'b1;
   // systemace_irq and systemace_mpbrdy are inputs

   // Logic Analyzer
   //assign analyzer1_data = 16'h0;
   //assign analyzer1_clock = 1'b1;
   assign analyzer1_clock = clock_27mhz;
   //assign analyzer2_data = 16'h0000;
   //assign analyzer2_clock = 1'b1;
   assign analyzer2_clock = tv_in_line_clock1;
   assign analyzer3_data = 16'h0;
   assign analyzer3_clock = 1'b1;
   assign analyzer4_data = 16'h0;
   assign analyzer4_clock = 1'b1;

   // VGA Output: this vga driver
   
   wire reset = ~button_enter;
   reg [15:0] vram_addr;
   reg [7:0] vram_data_in;
   wire [7:0] vram_data_out;
   reg vram_clk;
   reg vram_we;

   vga_with_ram vr(reset, clock_27mhz, vga_out_red, vga_out_green, 
		   vga_out_blue,
		   vga_out_sync_b, vga_out_blank_b, vga_out_pixel_clock, 
		   vga_out_hsync, vga_out_vsync,
		   vram_addr, vram_data_in, vram_data_out, vram_clk, vram_we);

   // ADV7185 NTSC decoder interface code
   	
   // adv7185 initialization module
   adv7185init adv7185(.reset(reset), .clock_27mhz(clock_27mhz), 
		       .source(1'b0), .tv_in_reset_b(tv_in_reset_b), 
		       .tv_in_i2c_clock(tv_in_i2c_clock), 
		       .tv_in_i2c_data(tv_in_i2c_data));

   wire [29:0] ycrcb;	// video data (luminance, chrominance)
   wire [2:0] fvh;	// sync for field, vertical, horizontal
   wire       dv;	// data valid
   
   ntsc_decode decode (.clk(tv_in_line_clock1), .reset(reset),
		       .tv_in_ycrcb(tv_in_ycrcb[19:10]), 
		       .ycrcb(ycrcb), .f(fvh[2]),
		       .v(fvh[1]), .h(fvh[0]), .data_valid(dv));

   // display memory: test pattern or NTSC video

   assign led = ~vram_data_out;

   wire [15:0] vaddr3;
   wire [7:0]  vdata3;
   wire        vclk3;
   wire        vwe3;
   vid_test_pat vp3 (clock_27mhz,vaddr3,vdata3,vclk3,vwe3);

   wire [15:0] vaddr4;
   wire [7:0]  vdata4;
   wire        vwe4;
   wire        vclk4;
   ntsc_to_ram vp4 (.clk(tv_in_line_clock1), 
		    .fvh(fvh), .dv(dv), .din(ycrcb[29:22]),
		    .vaddr(vaddr4), .vwe(vwe4), .vdata(vdata4));
   
   // select video source

   always @(switch[0])
     case (switch[0])
       1'b0: begin		// fill video RAM with NTSC video data
	  vram_addr = vaddr4;
	  vram_data_in = vdata4; // luminance data
	  vram_clk = tv_in_line_clock1; //vclk4;
	  vram_we = vwe4;
       end
       
       1'b1: begin 		// fill video RAM with b/w bars
	  vram_addr = vaddr3;
	  vram_data_in = vdata3; 
	  vram_clk = vclk3;
	  vram_we = vwe3;
       end
     endcase	  
   
   // debugging

   assign      analyzer2_data[15:0] = {2'b00, vwe4, fvh, tv_in_ycrcb[19:10]};

   parameter   VERSION = 8'd51;
   display_16hex my_display(reset, clock_27mhz, {56'b0,VERSION}, 
			    disp_blank, disp_clock, disp_rs, 
			    disp_ce_b, disp_reset_b, disp_data_out);
endmodule

// Fill video RAM with a video pattern: vertical bars

module vid_test_pat(clk,vaddr,vdata,vclk,vwe);

   output vclk,vwe;
   output [7:0] vdata;
   output [15:0] vaddr;
   input 	 clk;

   // generate 90 degree phase shifted signal for write-enable
   // so that we is stable and high on rising edge of count[0]

   reg [31:0] count;
   always @(posedge clk) count <= count + 1;

   reg 		 vwe;
   always @(posedge clk) vwe <= count[0] ^ vwe;

   assign     vaddr = count[17:2];
   assign     vclk = count[0];
   // assign     vwe = count[1];
   assign     vdata = { count[7:2], 2'b00 };

endmodule

// Fill video RAM from NTSC decoded video grabbed data 

module ntsc_to_ram(clk, fvh, dv, din, vdata, vaddr, vwe);

   output vwe;
   output [15:0] vaddr;
   output [7:0]  vdata;
   input [2:0] 	 fvh;
   input 	 dv;
   input [7:0] 	 din;
   input 	 clk;

   //   parameter 	 MAX_ROW = 191;
   //   parameter 	 MAX_COL = 255;
   
   // here put the luminance data from the ntsc decoder into the ram
   reg [7:0] 	 col = 0;
   reg [7:0] 	 row = 0;
   reg [7:0] 	 vdata = 0;
   reg 		 vwe = 0;
   
   always @ (posedge clk) //LLC1 is reference
     begin
	if (!fvh[2])
	  begin
	     col <= fvh[0] ? 8'h00 : 
		    (!fvh[2] && !fvh[1] && dv && (col < 255)) ? col + 1 : col;
	     row <= fvh[1] ? 8'h00 : 
		    (!fvh[2] && fvh[0] && (row < 191)) ? row + 1 : row;
	     vwe <= dv && !fvh[2]; // if the data is valid, we can write it
	     vdata <= (dv && !fvh[2]) ? din : vdata;
	  end
     end
   
   assign vaddr = {row,col};
   
endmodule

// SVGA video module with dual-port video ram
// the vram_* lines are used to read/write to the video RAM
// this module is configured to display a b&w 256x192 pixel image
// stored in the video RAM.

module vga_with_ram (reset, clock_27mhz, vga_out_red, vga_out_green, 
		     vga_out_blue,
		     vga_out_sync_b, vga_out_blank_b, vga_out_pixel_clock,
		     vga_out_hsync, vga_out_vsync, 
		     vram_addr, vram_data_in, vram_data_out, vram_clk, 
		     vram_we);

   input reset; // Active high reset, synchronous with 27MHz clock
   input clock_27mhz; // 27MHz input clock
   output [7:0] vga_out_red, vga_out_green, vga_out_blue; // Outputs to DAC
   output vga_out_sync_b, vga_out_blank_b; // Composite sync/blank outputs to DAC
   output vga_out_pixel_clock; // Pixel clock for DAC
   output vga_out_hsync, vga_out_vsync; // Sync outputs to VGA connector

   input [15:0] vram_addr;	// video ram address
   input [7:0] vram_data_in;	// video ram data input
   input  vram_clk, vram_we;	// video ram clock and write enable
   output [7:0] vram_data_out;	// video ram data output

   ////////////////////////////////////////////////////////////////////////////
   //
   // Timing values
   //
   ////////////////////////////////////////////////////////////////////////////
   
   // 1024 X 768 @ 75Hz with a 78.750MHz pixel clock

`define H_ACTIVE	1024 // pixels
`define H_FRONT_PORCH	  16 // pixels
`define H_SYNCH		  96 // pixels
`define H_BACK_PORCH	 176 // pixels
`define H_TOTAL		1312 // pixels
   
`define V_ACTIVE	 768 // lines
`define V_FRONT_PORCH	   1 // lines
`define V_SYNCH		   3 // lines
`define V_BACK_PORCH	  28 // lines
`define V_TOTAL		 800 // lines

   ////////////////////////////////////////////////////////////////////////////
   //
   // Internal signals
   //
   ////////////////////////////////////////////////////////////////////////////
   
   wire pixel_clock;
   reg prst, pixel_reset; // Active high reset, synchronous with pixel clock

   reg [7:0] vga_out_red, vga_out_blue, vga_out_green;
   wire vga_out_sync_b, vga_out_blank_b;	     
   reg hsync1, hsync2, vga_out_hsync, vsync1, vsync2, vga_out_vsync;
   
   reg [10:0] pixel_count; // Counts pixels in each line
   reg [10:0] line_count; // Counts lines in each frame

   reg [9:0] xpos;	// horizontal image pixel count
   reg [9:0] ypos;	// vertical image pixel count

   ////////////////////////////////////////////////////////////////////////////
   //
   // Generate the pixel clock (78.750MHz)
   //
   ////////////////////////////////////////////////////////////////////////////
   
   // synthesis attribute period of clock_27mhz is 37ns;
   
   DCM vga_dcm (.CLKIN(clock_27mhz), 
			.RST(1'b0),
			.CLKFX(pixel_clock));
   // synthesis attribute DLL_FREQUENCY_MODE of vga_dcm is "LOW"
   // synthesis attribute DUTY_CYCLE_CORRECTION of vga_dcm is "TRUE"
   // synthesis attribute STARTUP_WAIT of vga_dcm is "TRUE"
   // synthesis attribute DFS_FREQUENCY_MODE of vga_dcm is "LOW"
   // synthesis attribute CLKFX_DIVIDE of vga_dcm is 9
   // synthesis attribute CLKFX_MULTIPLY of vga_dcm is 26
   // synthesis attribute CLK_FEEDBACK of vga_dcm  is "NONE"
   // synthesis attribute CLKOUT_PHASE_SHIFT of vga_dcm is "NONE"
   // synthesis attribute PHASE_SHIFT of vga_dcm is 0
   // synthesis attribute clkin_period of vga_dcm is 37
   
   assign vga_out_pixel_clock = ~pixel_clock;

   always @(posedge pixel_clock)
     begin
	prst <= reset;
	pixel_reset <= prst;
     end
   
   ////////////////////////////////////////////////////////////////////////////
   //
   // Pixel and Line Counters
   //
   ////////////////////////////////////////////////////////////////////////////

   always @(posedge pixel_clock)
     if (pixel_reset)
       begin
	  pixel_count <= 0;
	  line_count <= 0;
       end
     else if (pixel_count == (`H_TOTAL-1)) // last pixel in the line
       begin
	  pixel_count <= 0;
	  if (line_count == (`V_TOTAL-1)) // last line of the frame
	    line_count <= 0;
	  else
	    line_count <= line_count + 1;
       end
     else
       pixel_count <= pixel_count +1;		

   always @(pixel_reset or pixel_count or line_count)
     if (pixel_reset)
       begin
	  xpos <= 0;
	  ypos <= 0;
       end
     else begin
        if (pixel_count > `H_FRONT_PORCH)
	  xpos <= pixel_count - `H_FRONT_PORCH;
        if (line_count > `V_FRONT_PORCH)
	  ypos <= line_count - `V_FRONT_PORCH;
     end

   ////////////////////////////////////////////////////////////////////////////
   //
   // Sync and Blank Signals
   //
   ////////////////////////////////////////////////////////////////////////////

   always @ (posedge pixel_clock)
     begin
	if (pixel_reset)
	  begin
	     hsync1 <= 1;
	     hsync2 <= 1;
	     vga_out_hsync <= 1;
	     vsync1 <= 1;
	     vsync2 <= 1;
	     vga_out_vsync <= 1;
	  end
	else 
	  begin

	     // Horizontal sync
	     if (pixel_count == (`H_ACTIVE+`H_FRONT_PORCH))
	       hsync1 <= 0; // start of h_sync
	     else if (pixel_count == (`H_ACTIVE+`H_FRONT_PORCH+`H_SYNCH))
	       hsync1 <= 1; // end of h_sync

	     // Vertical sync
	     if (pixel_count == (`H_TOTAL-1))
	       begin
		  if (line_count == (`V_ACTIVE+`V_FRONT_PORCH))
		    vsync1 <= 0; // start of v_sync
		  else if (line_count == (`V_ACTIVE+`V_FRONT_PORCH+`V_SYNCH))
		    vsync1 <= 1; // end of v_sync
	       end
	     
	  end

	// Delay hsync and vsync by two cycles to compensate for 2 cycles of
	// pipeline delay in the DAC.
	hsync2 <= hsync1;
	vga_out_hsync <= hsync2;
	vsync2 <= vsync1;
	vga_out_vsync <= vsync2;
		
     end

   // Blanking   
   assign vga_out_blank_b = ((pixel_count<`H_ACTIVE) & (line_count<`V_ACTIVE));

   // Composite sync
   assign vga_out_sync_b = hsync1 ^ vsync1;

   ////////////////////////////////////////////////////////////////////////////
   //
   // Display a 256x192 pixel image from dual-port RAM
   //
   ////////////////////////////////////////////////////////////////////////////

   reg [7:0] pixdata;	// for the memory read pipeline 
   reg [15:0] memaddr;	// for memory read address
   wire [7:0] memdata;  // for memory output data 8 bits: BBGGGRRR

   // read data from memory at pixel clock, with one pipeline stage

   always @(posedge pixel_clock)
     begin
      memaddr <= xpos[9:2] + ypos[9:2]*256;	// oversample
      pixdata <= memdata;		// latch in last value
     end

   // RAM with a 256 x 192 b&w image, 8-bits per pixel
   // the vga output is 1024x768, so we skip 2 bits of xpos and ypos, each

   videoram vram( .addra(memaddr), .clka(pixel_clock), .douta(memdata),
		  .addrb(vram_addr), 
		  .doutb(vram_data_out), .dinb(vram_data_in),
		  .clkb(vram_clk), .web(vram_we));

   // feed pixel values to VGA machine

   always @ (posedge pixel_clock)
     begin
	 /*
	vga_out_red   <= {pixdata[2:0], 5'b00000};
	vga_out_green <= {pixdata[5:3], 5'b00000};
	vga_out_blue  <= {pixdata[7:6], 6'b000000};
	   */
	 
	vga_out_red <= pixdata;		// for black and white display
	vga_out_green <= pixdata;
	vga_out_blue <= pixdata;
	   
     end
   
endmodule
